@@ -27,6 +27,7 @@
 #include "goo-application-actions-callbacks.h"
 #include "goo-window.h"
 #include "preferences.h"
+#include "main.h"
 
 
 void
@@ -61,7 +62,7 @@ goo_application_activate_about (GSimpleAction *action,
 
         gtk_show_about_dialog (GTK_WINDOW (_gtk_application_get_current_window (GTK_APPLICATION (user_data))),
                                "name", _("CD Player"),
-                               "version", VERSION,
+                               "version", PACKAGE_VERSION,
                                "copyright", _("Copyright \xc2\xa9 2004-2011 Free Software Foundation, Inc."),
                                "comments", _("Play CDs and save the tracks to disk as files"),
                                "authors", authors,
@@ -162,3 +163,48 @@ goo_application_activate_shuffle (GSimpleAction *action,
 
 	g_object_unref (settings);
 }
+
+
+void
+goo_application_activate_pause (GSimpleAction *action,
+				GVariant      *parameter,
+				gpointer       user_data)
+{
+	const char *device_id;
+	GtkWidget  *window;
+
+	device_id = g_variant_get_string (parameter, NULL);
+	window = main_get_window_from_device (device_id);
+	if (window != NULL)
+		goo_window_pause (GOO_WINDOW (window));
+}
+
+
+void
+goo_application_activate_play (GSimpleAction *action,
+			       GVariant      *parameter,
+			       gpointer       user_data)
+{
+	const char *device_id;
+	GtkWidget  *window;
+
+	device_id = g_variant_get_string (parameter, NULL);
+	window = main_get_window_from_device (device_id);
+	if (window != NULL)
+		goo_window_play (GOO_WINDOW (window));
+}
+
+
+void
+goo_application_activate_play_next (GSimpleAction *action,
+				    GVariant      *parameter,
+				    gpointer       user_data)
+{
+	const char *device_id;
+	GtkWidget  *window;
+
+	device_id = g_variant_get_string (parameter, NULL);
+	window = main_get_window_from_device (device_id);
+	if (window != NULL)
+		goo_window_next (GOO_WINDOW (window));
+}
