@@ -85,14 +85,15 @@ static const GOptionEntry options[] = {
 /* -- GooApplication --  */
 
 
-G_DEFINE_TYPE (GooApplication, goo_application, GTK_TYPE_APPLICATION)
-
-
 struct _GooApplicationPrivate {
-        GSettings *settings;
+	GSettings *settings;
 };
 
 
+G_DEFINE_TYPE_WITH_CODE (GooApplication, goo_application, GTK_TYPE_APPLICATION,
+			 G_ADD_PRIVATE (GooApplication))
+
+
 static void
 goo_application_finalize (GObject *object)
 {
@@ -196,13 +197,12 @@ goo_application_command_line (GApplication            *application,
 
 	if (! required_gstreamer_plugins_available ()) {
 		GtkWidget *d;
-		d = _gtk_message_dialog_new (NULL,
-					     0,
-					     _GTK_ICON_NAME_DIALOG_ERROR,
-					     _("Cannot start the CD player"),
-					     _("In order to read CDs you have to install the gstreamer base plugins"),
-					     _GTK_LABEL_OK, GTK_RESPONSE_OK,
-					     NULL);
+		d = _gtk_error_dialog_new (NULL,
+					   0,
+					   NULL,
+					   _("Cannot start the CD player"),
+					   "%s",
+					   _("In order to read CDs you have to install the GStreamer base plugins"));
 		g_signal_connect_swapped (G_OBJECT (d), "response",
 					  G_CALLBACK (gtk_widget_destroy),
 					  d);
@@ -346,17 +346,18 @@ pref_playlist_shuffle_changed (GSettings  *settings,
 static void
 initialize_app_menu (GApplication *application)
 {
+	const _GtkAccelerator app_accelerators[] = {
+		{ "app.help", "F1" },
+		{ "app.quit", "<Control>q" }
+	};
 	GooApplication *self = (GooApplication *) application;
-	GtkBuilder     *builder;
 
 	g_action_map_add_action_entries (G_ACTION_MAP (application),
 					 goo_application_actions,
 					 G_N_ELEMENTS (goo_application_actions),
 					 application);
 
-	builder = _gtk_builder_new_from_resource ("app-menu.ui");
-	gtk_application_set_app_menu (GTK_APPLICATION (application),
-				      G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
+	_gtk_application_add_accelerators (GTK_APPLICATION (application), app_accelerators, G_N_ELEMENTS (app_accelerators));
 
 	g_simple_action_set_state (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_PLAYALL)),
 				   g_variant_new_boolean (g_settings_get_boolean (self->priv->settings, PREF_PLAYLIST_PLAYALL)));
@@ -377,8 +378,6 @@ initialize_app_menu (GApplication *application)
 			  "changed::" PREF_PLAYLIST_REPEAT,
 			  G_CALLBACK (pref_playlist_repeat_changed),
 			  self);
-
-	g_object_unref (builder);
 }
 
 
@@ -394,14 +393,23 @@ goo_application_startup (GApplication *application)
 }
 
 
+static void
+goo_application_activate (GApplication *application)
+{
+	GtkWidget *window;
+
+	window = _gtk_application_get_current_window (GTK_APPLICATION (application));
+	if (window != NULL)
+		gtk_window_present (GTK_WINDOW (window));
+}
+
+
 static void
 goo_application_class_init (GooApplicationClass *klass)
 {
 	GObjectClass      *object_class;
 	GApplicationClass *application_class;
 
-	g_type_class_add_private (klass, sizeof (GooApplicationPrivate));
-
 	object_class = G_OBJECT_CLASS (klass);
 	object_class->finalize = goo_application_finalize;
 
@@ -409,13 +417,14 @@ goo_application_class_init (GooApplicationClass *klass)
 	application_class->command_line = goo_application_command_line;
 	application_class->local_command_line = goo_application_local_command_line;
 	application_class->startup = goo_application_startup;
+	application_class->activate = goo_application_activate;
 }
 
 
 static void
 goo_application_init (GooApplication *self)
 {
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GOO_TYPE_APPLICATION, GooApplicationPrivate);
+	self->priv = goo_application_get_instance_private (self);
 	self->priv->settings = g_settings_new (GOOBOX_SCHEMA_PLAYLIST);
 }
 
