@@ -38,6 +38,8 @@
 #define REFRESH_RATE 5
 #define PROGRESS_DELAY 400
 #define QUEUE_SIZE 16384U /*131072U*/
+#define PIPELINE_VOLUME(x) ((x) / 100.0)
+
 
 struct _GooPlayerPrivate {
 	BraseroDrive    *drive;
@@ -80,10 +82,9 @@ static guint goo_player_signals[LAST_SIGNAL] = { 0 };
 
 static void goo_player_finalize    (GObject *object);
 
-#define GOO_PLAYER_GET_PRIVATE_DATA(object) \
-	(G_TYPE_INSTANCE_GET_PRIVATE ((object), GOO_TYPE_PLAYER, GooPlayerPrivate))
 
-G_DEFINE_TYPE (GooPlayer, goo_player, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_CODE (GooPlayer, goo_player, G_TYPE_OBJECT,
+			 G_ADD_PRIVATE (GooPlayer))
 
 
 static void
@@ -283,6 +284,8 @@ create_pipeline (GooPlayer *self)
 		      "vis-plugin", vis_plugin,
 		      "flags", flags,
 		      "uri", "cdda://",
+		      "volume", PIPELINE_VOLUME (self->priv->volume_value),
+		      "buffer-duration", (guint64) 10 * GST_SECOND,
 		      NULL);
 
 	g_signal_connect (self->priv->pipeline,
@@ -318,13 +321,6 @@ goo_player_set_state (GooPlayer       *self,
 		      GooPlayerState   state,
 		      gboolean         notify)
 {
-	if (self->priv->drive != NULL) {
-		if (state == GOO_PLAYER_STATE_PLAYING)
-			brasero_drive_lock (self->priv->drive, _("Playing CD"), NULL);
-		else
-			brasero_drive_unlock (self->priv->drive);
-	}
-
 	self->priv->state = state;
 	if (notify)
 		g_signal_emit (G_OBJECT (self),
@@ -342,8 +338,6 @@ goo_player_class_init (GooPlayerClass *class)
 	gobject_class = G_OBJECT_CLASS (class);
 	gobject_class->finalize = goo_player_finalize;
 
-	g_type_class_add_private (class, sizeof (GooPlayerPrivate));
-
 	goo_player_signals[START] =
                 g_signal_new ("start",
 			      G_TYPE_FROM_CLASS (class),
@@ -396,7 +390,7 @@ goo_player_class_init (GooPlayerClass *class)
 static void
 goo_player_init (GooPlayer *self)
 {
-	self->priv = GOO_PLAYER_GET_PRIVATE_DATA (self);
+	self->priv = goo_player_get_instance_private (self);
 	self->priv->state = GOO_PLAYER_STATE_NO_DISC;
 	self->priv->action = GOO_PLAYER_ACTION_NONE;
 	self->priv->is_busy = FALSE;
@@ -428,7 +422,6 @@ goo_player_finalize (GObject *object)
 	self->priv->exiting = TRUE;
         g_mutex_unlock (&self->priv->data_mutex);
 
-        brasero_drive_unlock (self->priv->drive);
         if (self->priv->medium_added_event != 0)
 		g_signal_handler_disconnect (self->priv->drive, self->priv->medium_added_event);
 	if (self->priv->medium_removed_event != 0)
@@ -605,7 +598,7 @@ get_cd_info_from_device_ready_cb (GObject      *source_object,
 				  gpointer      user_data)
 {
 	GooPlayer *player = user_data;
-	AlbumInfo *album;
+	AlbumInfo *album = NULL;
 	GError    *error = NULL;
 
 	if (metadata_get_cd_info_from_device_finish (result,
@@ -964,10 +957,8 @@ goo_player_set_audio_volume (GooPlayer *player,
 		return;
 
 	player->priv->volume_value = vol;
-	/*
 	if (player->priv->pipeline != NULL)
-		g_object_set (G_OBJECT (player->priv->pipeline), "volume", vol, NULL);
-	*/
+		g_object_set (G_OBJECT (player->priv->pipeline), "volume", PIPELINE_VOLUME (player->priv->volume_value), NULL);
 }
 
 
