@@ -31,67 +31,30 @@
 
 
 GtkWidget *
-_gtk_message_dialog_new (GtkWindow        *parent,
-			 GtkDialogFlags    flags,
-			 const char       *icon_name,
-			 const char       *message,
-			 const char       *secondary_message,
-			 const char       *first_button_text,
+_gtk_message_dialog_new (GtkWindow      *parent,
+			 GtkDialogFlags  flags,
+			 const char     *message,
+			 const char     *secondary_message,
+			 const gchar    *first_button_text,
 			 ...)
 {
-	GtkBuilder  *builder;
 	GtkWidget   *dialog;
-	GtkWidget   *label;
 	va_list      args;
 	const gchar *text;
 	int          response_id;
-	char        *markup_text;
 
-	builder = _gtk_builder_new_from_resource ("message-dialog.ui");
-	dialog = _gtk_builder_get_widget (builder, "message_dialog");
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
-	gtk_window_set_modal (GTK_WINDOW (dialog), (flags & GTK_DIALOG_MODAL));
-	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), (flags & GTK_DIALOG_DESTROY_WITH_PARENT));
-	g_object_set_data_full (G_OBJECT (dialog), "builder", builder, g_object_unref);
+	dialog = gtk_message_dialog_new (parent,
+					 flags,
+					 GTK_MESSAGE_OTHER,
+					 GTK_BUTTONS_NONE,
+					 "%s", message);
+
+	if (secondary_message != NULL)
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", secondary_message);
 
 	if (flags & GTK_DIALOG_MODAL)
 		_gtk_dialog_add_to_window_group (GTK_DIALOG (dialog));
 
-	/* set the icon */
-
-	gtk_image_set_from_icon_name (GTK_IMAGE (_gtk_builder_get_widget (builder, "icon_image")),
-				      icon_name,
-				      GTK_ICON_SIZE_DIALOG);
-
-	/* set the message */
-
-	label = _gtk_builder_get_widget (builder, "message_label");
-
-	if (message != NULL) {
-		char *escaped_message;
-
-		escaped_message = g_markup_escape_text (message, -1);
-		if (secondary_message != NULL) {
-			char *escaped_secondary_message;
-
-			escaped_secondary_message = g_markup_escape_text (secondary_message, -1);
-			markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
-						       escaped_message,
-						       escaped_secondary_message);
-
-			g_free (escaped_secondary_message);
-		}
-		else
-			markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>", escaped_message);
-
-		g_free (escaped_message);
-	}
-	else
-		markup_text = g_markup_escape_text (secondary_message, -1);
-
-	gtk_label_set_markup (GTK_LABEL (label), markup_text);
-	g_free (markup_text);
-
 	/* add the buttons */
 
 	if (first_button_text == NULL)
@@ -117,74 +80,95 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 }
 
 
-/* -- _gtk_ok_dialog_with_checkbutton_new -- */
-
-
-typedef struct {
-	GSettings *settings;
-	char      *key;
-} DialogWithButtonData;
-
-
-static void
-dialog_with_button_data_free (DialogWithButtonData *data)
+GtkWidget *
+_gtk_error_dialog_new (GtkWindow      *parent,
+		       GtkDialogFlags  flags,
+		       GList          *row_output,
+		       const char     *primary_text,
+		       const char     *secondary_text_format,
+		       ...)
 {
-	_g_object_unref (data->settings);
-	g_free (data->key);
-	g_free (data);
-}
+	GtkWidget *dialog;
 
+	dialog = gtk_message_dialog_new (parent,
+					 flags,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_CLOSE,
+					 "%s", primary_text);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
-static void
-ok__check_button_toggled_cb (GtkToggleButton      *button,
-			     DialogWithButtonData *data)
-{
-	g_settings_set_boolean (data->settings, data->key, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
-}
+	if (flags & GTK_DIALOG_MODAL)
+		_gtk_dialog_add_to_window_group (GTK_DIALOG (dialog));
 
+	/* label */
 
-GtkWidget*
-_gtk_ok_dialog_with_checkbutton_new (GtkWindow        *parent,
-				     GtkDialogFlags    flags,
-				     const char       *message,
-				     const char       *ok_button_text,
-				     const char       *check_button_label,
-				     GSettings        *settings,
-				     const char       *key)
-{
-	GtkWidget		*d;
-	GtkBuilder              *builder;
-	GtkWidget		*check_button;
-	DialogWithButtonData	*data;
+	if (secondary_text_format != NULL) {
+		va_list  args;
+		char    *secondary_message;
 
-	d = _gtk_message_dialog_new (parent,
-				     flags,
-				     _GTK_ICON_NAME_DIALOG_INFO,
-				     message,
-				     NULL,
-				     ok_button_text, GTK_RESPONSE_OK,
-				     NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_OK);
+		va_start (args, secondary_text_format);
+		secondary_message = g_strdup_vprintf (secondary_text_format, args);
+		va_end (args);
 
-	/* setup the checkbutton */
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", secondary_message);
 
-	builder = g_object_get_data (G_OBJECT (d), "builder");
-	check_button = _gtk_builder_get_widget(builder, "message_checkbutton");
-	gtk_container_add (GTK_CONTAINER (check_button), gtk_label_new_with_mnemonic (check_button_label));
-	gtk_widget_show_all (check_button);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button), g_settings_get_boolean (settings, key));
+		g_free (secondary_message);
+	}
 
-	data = g_new0 (DialogWithButtonData, 1);
-	data->settings = g_object_ref (settings);
-	data->key = g_strdup (key);
-	g_object_set_data_full (G_OBJECT (d), "settings-data", data, (GDestroyNotify) dialog_with_button_data_free);
+	/* output */
 
-	g_signal_connect (G_OBJECT (check_button),
-			  "toggled",
-			  G_CALLBACK (ok__check_button_toggled_cb),
-			  data);
+	if ((row_output != NULL) && (secondary_text_format == NULL)) {
+		GtkWidget     *output_box;
+		GtkWidget     *label;
+		GtkWidget     *scrolled_window;
+		GtkWidget     *text_view;
+		GtkTextBuffer *text_buffer;
+		GtkTextIter    iter;
+		GList         *scan;
 
-	return d;
+		output_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+		gtk_box_pack_end (GTK_BOX (gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog))),
+				  output_box, TRUE, TRUE, 0);
+
+		label = gtk_label_new_with_mnemonic (_("C_ommand Line Output:"));
+		gtk_box_pack_start (GTK_BOX (output_box), label, FALSE, FALSE, 0);
+
+		scrolled_window = g_object_new (GTK_TYPE_SCROLLED_WINDOW,
+						"shadow-type", GTK_SHADOW_IN,
+						"width-request", 450,
+						"height-request", 200,
+						NULL);
+		gtk_box_pack_start (GTK_BOX (output_box), scrolled_window, TRUE, TRUE, 0);
+
+		text_view = gtk_text_view_new ();
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), text_view);
+		gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
+
+		text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
+		gtk_text_buffer_create_tag (text_buffer, "monospace",
+					    "family", "monospace",
+					    NULL);
+		gtk_text_buffer_get_iter_at_offset (text_buffer, &iter, 0);
+		for (scan = row_output; scan; scan = scan->next) {
+			char  *line = scan->data;
+			char  *utf8_line;
+			gsize  bytes_written;
+
+			utf8_line = g_locale_to_utf8 (line, -1, NULL, &bytes_written, NULL);
+			gtk_text_buffer_insert_with_tags_by_name (text_buffer,
+								  &iter,
+								  utf8_line,
+								  bytes_written,
+								  "monospace", NULL);
+			g_free (utf8_line);
+
+			gtk_text_buffer_insert (text_buffer, &iter, "\n", 1);
+		}
+
+		gtk_widget_show_all (output_box);
+	}
+
+	return dialog;
 }
 
 
@@ -197,13 +181,12 @@ _gtk_error_dialog_from_gerror_run (GtkWindow   *parent,
 
 	g_return_if_fail (*gerror != NULL);
 
-	d = _gtk_message_dialog_new (parent,
-				     GTK_DIALOG_DESTROY_WITH_PARENT,
-				     _GTK_ICON_NAME_DIALOG_ERROR,
-				     title,
-				     (*gerror)->message,
-				     _GTK_LABEL_OK, GTK_RESPONSE_OK,
-				     NULL);
+	d = _gtk_error_dialog_new (parent,
+				   GTK_DIALOG_DESTROY_WITH_PARENT,
+				   NULL,
+				   title,
+				   "%s",
+				   (*gerror)->message);
 	gtk_dialog_run (GTK_DIALOG (d));
 
 	gtk_widget_destroy (d);
@@ -227,13 +210,12 @@ _gtk_error_dialog_from_gerror_show (GtkWindow   *parent,
 {
 	GtkWidget *d;
 
-	d = _gtk_message_dialog_new (parent,
-				     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				     _GTK_ICON_NAME_DIALOG_ERROR,
-				     title,
-				     (gerror != NULL) ? (*gerror)->message : NULL,
-				     _GTK_LABEL_OK, GTK_RESPONSE_OK,
-				     NULL);
+	d = _gtk_error_dialog_new (parent,
+				   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				   NULL,
+				   title,
+				   (gerror != NULL) ? "%s" : NULL,
+				   (*gerror)->message);
 	g_signal_connect (d, "response", G_CALLBACK (error_dialog_response_cb), NULL);
 
 	gtk_window_present (GTK_WINDOW (d));
@@ -243,67 +225,6 @@ _gtk_error_dialog_from_gerror_show (GtkWindow   *parent,
 }
 
 
-
-void
-_gtk_error_dialog_run (GtkWindow        *parent,
-		       const gchar      *format,
-		       ...)
-{
-	GtkWidget *d;
-	char      *message;
-	va_list    args;
-
-	va_start (args, format);
-	message = g_strdup_vprintf (format, args);
-	va_end (args);
-
-	d =  _gtk_message_dialog_new (parent,
-				      GTK_DIALOG_MODAL,
-				      _GTK_ICON_NAME_DIALOG_ERROR,
-				      message,
-				      NULL,
-				      _GTK_LABEL_CLOSE, GTK_RESPONSE_CANCEL,
-				      NULL);
-	g_free (message);
-
-	g_signal_connect (G_OBJECT (d), "response",
-			  G_CALLBACK (gtk_widget_destroy),
-			  NULL);
-
-	gtk_widget_show (d);
-}
-
-
-void
-_gtk_info_dialog_run (GtkWindow        *parent,
-		      const gchar      *format,
-		      ...)
-{
-	GtkWidget *d;
-	char      *message;
-	va_list    args;
-
-	va_start (args, format);
-	message = g_strdup_vprintf (format, args);
-	va_end (args);
-
-	d =  _gtk_message_dialog_new (parent,
-				      GTK_DIALOG_MODAL,
-				      _GTK_ICON_NAME_DIALOG_INFO,
-				      message,
-				      NULL,
-				      _GTK_LABEL_CLOSE, GTK_RESPONSE_CANCEL,
-				      NULL);
-	g_free (message);
-
-	g_signal_connect (G_OBJECT (d), "response",
-			  G_CALLBACK (gtk_widget_destroy),
-			  NULL);
-
-	gtk_widget_show (d);
-}
-
-
 void
 _gtk_dialog_add_to_window_group (GtkDialog *dialog)
 {
@@ -491,24 +412,20 @@ show_help_dialog (GtkWindow  *parent,
 	GError *error = NULL;
 
 	uri = g_strconcat ("help:goobox", section ? "?" : NULL, section, NULL);
-	if (! gtk_show_uri (gtk_window_get_screen (parent), uri, GDK_CURRENT_TIME, &error)) {
+	if (! gtk_show_uri_on_window (parent, uri, GDK_CURRENT_TIME, &error)) {
   		GtkWidget *dialog;
 
-		dialog = _gtk_message_dialog_new (parent,
-						  GTK_DIALOG_DESTROY_WITH_PARENT,
-						  _GTK_ICON_NAME_DIALOG_ERROR,
-						  _("Could not display help"),
-						  error->message,
-						  _GTK_LABEL_OK, GTK_RESPONSE_OK,
-						  NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+		dialog = _gtk_error_dialog_new (parent,
+						GTK_DIALOG_DESTROY_WITH_PARENT,
+						NULL,
+						_("Could not display help"),
+						"%s",
+						error->message);
 
 		g_signal_connect (G_OBJECT (dialog), "response",
 				  G_CALLBACK (gtk_widget_destroy),
 				  NULL);
 
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-
 		gtk_widget_show (dialog);
 
 		g_clear_error (&error);
@@ -685,40 +602,6 @@ _gtk_image_new_from_xpm_data (char * xpm_data[])
 }
 
 
-GtkWidget *
-_gtk_image_new_from_inline (const guint8 *data)
-{
-	GdkPixbuf *pixbuf;
-	GtkWidget *image;
-
-	pixbuf = gdk_pixbuf_new_from_inline (-1, data, FALSE, NULL);
-	image = gtk_image_new_from_pixbuf (pixbuf);
-	gtk_widget_show (image);
-
-	g_object_unref (G_OBJECT (pixbuf));
-
-	return image;
-}
-
-
-void
-_gtk_widget_get_screen_size (GtkWidget *widget,
-			     int       *width,
-			     int       *height)
-{
-	GdkScreen    *screen;
-	GdkRectangle  screen_geom;
-
-	screen = gtk_widget_get_screen (widget);
-	gdk_screen_get_monitor_geometry (screen,
-					 gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (widget)),
-					 &screen_geom);
-
-	*width = screen_geom.width;
-	*height = screen_geom.height;
-}
-
-
 void
 _gtk_tree_path_list_free (GList *list)
 {
@@ -1039,3 +922,136 @@ _gtk_application_get_current_window (GtkApplication *application)
 
 	return GTK_WIDGET (windows->data);
 }
+
+
+void
+_gtk_application_add_accelerator_for_action (GtkApplication   *app,
+					     const char       *action_name,
+					     const char       *accel)
+{
+	const char *accels[2];
+
+	accels[0] = accel;
+	accels[1] = NULL;
+	gtk_application_set_accels_for_action (app, action_name, accels);
+}
+
+
+void
+_gtk_application_add_accelerators (GtkApplication        *app,
+				   const _GtkAccelerator *accelerators,
+				   int                    n_accelerators)
+{
+	int i;
+
+	for (i = 0; i < n_accelerators; i++) {
+		const _GtkAccelerator *acc = accelerators + i;
+		_gtk_application_add_accelerator_for_action (GTK_APPLICATION (app),
+							     acc->action_name,
+							     acc->accelerator);
+	}
+}
+
+
+gboolean
+_gtk_window_get_monitor_info (GtkWindow	    *window,
+			      GdkRectangle  *geometry,
+			      int           *number,
+			      char         **name)
+{
+#if GTK_CHECK_VERSION(3, 22, 0)
+
+	GdkWindow  *win;
+	GdkMonitor *monitor;
+
+	win = gtk_widget_get_window (GTK_WIDGET (window));
+	if (win == NULL)
+		return FALSE;
+
+	monitor = gdk_display_get_monitor_at_window (gdk_window_get_display (win), win);
+	if (monitor == NULL)
+		return FALSE;
+
+	if (geometry != NULL)
+		gdk_monitor_get_geometry (monitor, geometry);
+
+	if ((number != NULL) || (name != NULL)) {
+		GdkDisplay *display;
+		int         monitor_num;
+		const char *monitor_name;
+		int         i;
+
+		display = gdk_monitor_get_display (monitor);
+		monitor_num = 0;
+		for (i = 0; /* void */; i++) {
+			GdkMonitor *m = gdk_display_get_monitor (display, i);
+			if (m == monitor) {
+				monitor_num = i;
+				monitor_name = gdk_monitor_get_model (monitor);
+				break;
+			}
+			if (m == NULL)
+				break;
+		}
+
+		if (number != NULL) *number = monitor_num;
+		if (name != NULL) *name = g_strdup (monitor_name);
+	}
+
+#else
+
+	GdkWindow *win;
+	GdkScreen *screen;
+	int        monitor_num;
+
+	win = gtk_widget_get_window (GTK_WIDGET (window));
+	if (win == NULL)
+		return FALSE;
+
+	screen = gdk_window_get_screen (win);
+	if (screen == NULL)
+		return FALSE;
+
+	monitor_num = gdk_screen_get_monitor_at_window (screen, win);
+	if (number != NULL)
+		*number = monitor_num;
+	if (geometry != NULL)
+		gdk_screen_get_monitor_geometry (screen, monitor_num, geometry);
+	if (name != NULL)
+		*name = gdk_screen_get_monitor_plug_name (screen, monitor_num);
+
+#endif
+
+	return TRUE;
+}
+
+
+gboolean
+_gtk_widget_get_monitor_geometry (GtkWidget    *widget,
+				  GdkRectangle *geometry)
+{
+	gboolean   result = FALSE;
+	GtkWidget *window;
+
+	window = gtk_widget_get_toplevel (widget);
+	if (GTK_IS_WINDOW (window)) {
+		if (_gtk_window_get_monitor_info (GTK_WINDOW (window), geometry, NULL, NULL)) {
+			result = TRUE;
+		}
+	}
+
+	return result;
+}
+
+
+gboolean
+_gtk_settings_get_dialogs_use_header (void)
+{
+	gboolean use_header;
+
+	g_object_get (gtk_settings_get_default (),
+		      "gtk-dialogs-use-header", &use_header,
+		      NULL);
+
+	return use_header;
+}
