@@ -30,12 +30,12 @@ extern GtkApplication *Main_Application;
 extern int	       arg_auto_play;
 extern int	       arg_toggle_visibility;
 
-GtkWidget *     main_get_window_from_device  (const char      *device);
-BraseroDrive *  main_get_most_likely_drive   (void);
-BraseroDrive *  main_get_drive_for_device    (const char      *device);
-gboolean        notification_has_persistence (void);
-void            system_notify                (GooWindow       *window,
-					      const char      *title,
-					      const char      *msg);
+GtkWidget *	main_get_window_from_device	(const char	*device);
+BraseroDrive *	main_get_most_likely_drive	(void);
+BraseroDrive *	main_get_drive_for_device	(const char	*device);
+void		system_notify			(GooWindow	*window,
+						const char	*id,
+						const char	*title,
+						const char	*msg);
 
 #endif /* MAIN_H */
